'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var path = require('path');

var eslint = require('eslint');
var isAbsolute = require('path-is-absolute');
var difference = require('./array-diff');
var getSortedRules = require('./sort-rules');

function _getConfigFile(specifiedFile) {
  if (specifiedFile) {
    if (isAbsolute(specifiedFile)) {
      return specifiedFile;
    }
    return path.join(process.cwd(), specifiedFile); // eslint-disable-line import/no-dynamic-require
  }
  // This is not being called with an arg. Use the package.json `main`
  return require(path.join(process.cwd(), 'package.json')).main; // eslint-disable-line import/no-dynamic-require
}

function _getConfig(configFile) {
  var cliEngine = new eslint.CLIEngine({
    // Ignore any config applicable depending on the location on the filesystem
    useEslintrc: false,
    // Point to the particular config
    configFile: configFile
  });
  return cliEngine.getConfigForFile();
}

function _getCurrentRules(config) {
  return Object.keys(config.rules);
}

function _normalizePluginName(name) {
  var scopedRegex = /(@[^/]+)\/(.+)/;
  var match = scopedRegex.exec(name);

  if (match) {
    return {
      module: match[1] + '/eslint-plugin-' + match[2],
      prefix: match[2]
    };
  }

  return {
    module: 'eslint-plugin-' + name,
    prefix: name
  };
}

function _isDeprecated(rule) {
  return rule.meta && rule.meta.deprecated;
}

function _getPluginRules(config, _ref) {
  var includeDeprecated = _ref.includeDeprecated;

  var pluginRules = [];
  var plugins = config.plugins;
  if (plugins) {
    plugins.forEach(function (plugin) {
      var normalized = _normalizePluginName(plugin);
      var pluginConfig = require(normalized.module); // eslint-disable-line import/no-dynamic-require

      var rules = pluginConfig.rules === undefined ? {} : pluginConfig.rules;
      pluginRules = pluginRules.concat(Object.keys(rules).filter(function (rule) {
        return includeDeprecated || !_isDeprecated(rules[rule]);
      }).map(function (rule) {
        return normalized.prefix + '/' + rule;
      }));
    });
  }
  return pluginRules;
}

function _getCoreRules(_ref2) {
  var includeDeprecated = _ref2.includeDeprecated;

  var rules = eslint.linter.getRules();
  return Array.from(rules.keys()).filter(function (rule) {
    return includeDeprecated || !_isDeprecated(rules.get(rule));
  });
}

function _isNotCore(rule) {
  return rule.indexOf('/') !== '-1';
}

function RuleFinder(specifiedFile, options) {
  var omitCore = options.omitCore,
      includeDeprecated = options.includeDeprecated;

  var configFile = _getConfigFile(specifiedFile);
  var config = _getConfig(configFile);
  var currentRules = _getCurrentRules(config);
  var pluginRules = _getPluginRules(config, { includeDeprecated: includeDeprecated });
  var coreRules = _getCoreRules({ includeDeprecated: includeDeprecated });
  var allRules = omitCore ? pluginRules : [].concat(_toConsumableArray(coreRules), _toConsumableArray(pluginRules));
  if (omitCore) {
    currentRules = currentRules.filter(_isNotCore);
  }
  var unusedRules = difference(allRules, currentRules); // eslint-disable-line vars-on-top

  // Get all the current rules instead of referring the extended files or documentation
  this.getCurrentRules = function () {
    return getSortedRules(currentRules);
  };

  // Get all the current rules' particular configuration
  this.getCurrentRulesDetailed = function () {
    return config.rules;
  };

  // Get all the plugin rules instead of referring the extended files or documentation
  this.getPluginRules = function () {
    return getSortedRules(pluginRules);
  };

  // Get all the available rules instead of referring eslint and plugin packages or documentation
  this.getAllAvailableRules = function () {
    return getSortedRules(allRules);
  };

  this.getUnusedRules = function () {
    return getSortedRules(unusedRules);
  };
}

module.exports = function (specifiedFile) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return new RuleFinder(specifiedFile, options);
};